@@ -1,7 +1,12 @@
+#![no_std]
+
 /** References: https://www.masswerk.at/6502/6502_instruction_set.html
 */
-use take_mut;
-use yaxpeax_arch::{AddressDiff, Arch, Decoder, LengthedInstruction};
+
+#[cfg(feature = "std")]
+extern crate std;
+
+use yaxpeax_arch::{AddressDiff, Arch, Decoder, LengthedInstruction, Reader};
 
 mod display;
 
@@ -10,6 +15,7 @@ pub struct N6502;
 
 impl Arch for N6502 {
     type Address = u16;
+    type Word = u8;
     type Instruction = Instruction;
     type DecodeError = DecodeError;
     type Decoder = InstDecoder;
@@ -156,24 +162,7 @@ impl Operand {
     }
 }
 
-#[derive(Debug, PartialEq)]
-pub enum DecodeError {
-    ExhaustedInput,
-    InvalidOpcode,
-    InvalidOperand,
-}
-
-impl yaxpeax_arch::DecodeError for DecodeError {
-    fn data_exhausted(&self) -> bool {
-        self == &DecodeError::ExhaustedInput
-    }
-    fn bad_opcode(&self) -> bool {
-        self == &DecodeError::InvalidOpcode
-    }
-    fn bad_operand(&self) -> bool {
-        self == &DecodeError::InvalidOperand
-    }
-}
+pub type DecodeError = yaxpeax_arch::StandardDecodeError;
 
 #[derive(Debug)]
 pub struct InstDecoder;
@@ -363,18 +352,15 @@ impl Default for InstDecoder {
     }
 }
 
-impl Decoder<Instruction> for InstDecoder {
-    type Error = DecodeError;
-
-    fn decode_into<T: IntoIterator<Item = u8>>(
+impl Decoder<N6502> for InstDecoder {
+    fn decode_into<T: Reader<<N6502 as Arch>::Address, <N6502 as Arch>::Word>>(
         &self,
         inst: &mut Instruction,
-        bytes: T,
-    ) -> Result<(), Self::Error> {
-        let mut bytes_iter = bytes.into_iter();
-        let opcode = bytes_iter.next().ok_or(DecodeError::ExhaustedInput)?;
+        words: &mut T,
+    ) -> Result<(), <N6502 as Arch>::DecodeError> {
+        let opcode = words.next()?;
 
-        let (op_type, mut operand) = self.op_type(opcode).map_err(|e| {
+        let (op_type, operand) = self.op_type(opcode).map_err(|e| {
             inst.opcode = Opcode::Invalid(opcode);
             e
         })?;
@@ -385,11 +371,11 @@ impl Decoder<Instruction> for InstDecoder {
         match operand.width() {
             0 => {}
             1 => {
-                op_byte = bytes_iter.next().ok_or(DecodeError::ExhaustedInput)?;
+                op_byte = words.next()?;
             }
             2 => {
-                let byte_lo = bytes_iter.next().ok_or(DecodeError::ExhaustedInput)?;
-                let byte_hi = bytes_iter.next().ok_or(DecodeError::ExhaustedInput)?;
+                let byte_lo = words.next()?;
+                let byte_hi = words.next()?;
 
                 op_word = u16::from_le_bytes([byte_lo, byte_hi]);
             }
@@ -398,7 +384,7 @@ impl Decoder<Instruction> for InstDecoder {
             }
         }
 
-        take_mut::take(&mut operand, |op| match op {
+        let operand = match operand {
             Operand::Accumulator => Operand::Accumulator,
             Operand::Implied => Operand::Implied,
 
@@ -414,7 +400,7 @@ impl Decoder<Instruction> for InstDecoder {
             Operand::AbsoluteX(_) => Operand::AbsoluteX(op_word),
             Operand::AbsoluteY(_) => Operand::AbsoluteY(op_word),
             Operand::Indirect(_) => Operand::Indirect(op_word),
-        });
+        };
 
         inst.opcode = op_type;
         inst.operand = operand;
